@@ -3,47 +3,58 @@
  * This file provides optimization functions for the Scythe
  * Statistical Library.
  *
- * Scythe C++ Library
- * Copyright (C) Kevin M. Quinn, Andrew D. Martin,
- * and Daniel B. Pemstein
+ * Scythe Statistical Library
+ * Copyright (C) 2003, Andrew D. Martin, Kevin M. Quinn, and Daniel
+ * Pemstein.  All Rights Reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or (at
+ * your option) any later version.  A copy of this license is included
+ * with this library (LICENSE.GPL).
+ *
+ * This library utilizes code from a number of other open source
+ * projects.  Specific copyright information is provided with the
+ * applicable code.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * USA.
  *
  * This code written by:
  *
- * Kevin Quinn
- * Assistant Professor
- * Dept. of Political Science and
- * Center for Statistics and Social Sciences
- * Box 354322
- * University of Washington
- * Seattle, WA 98195-4322
- * quinn@stat.washington.edu
- *
  * Andrew D. Martin
  * Assistant Professor
- * Dept. of Political Science
+ * Deptartment of Political Science
  * Campus Box 1063
  * Washington University
+ * One Brookings Drive
  * St. Louis, MO 63130
  * admartin@artsci.wustl.edu
- * 
- * Daniel B. Pemstein
- * dbpemste@artsci.wustl.edu
- * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ *
+ * Kevin M. Quinn
+ * Assistant Professor
+ * Department of Government and
+ * Center for Basic Research in the Social Sciences
+ * 34 Kirkland Street
+ * Harvard University
+ * Cambridge, MA 02138
+ * kquinn@fas.harvard.edu
+ *
+ * Daniel Pemstein
+ * Deptartment of Poltical Science
+ * 702 South Wright Street
+ * University of Illinois at Urbana-Champaign
+ * Urbana, IL 61801
+ * dbp@uiuc.edu
  */
+
 #ifndef SCYTHE_OPTIMIZE_CC
 #define SCYTHE_OPTIMIZE_CC
 
@@ -79,18 +90,18 @@ namespace SCYTHE {
   epsilon()
   {
     T eps, del, neweps;
-    del 	 = (T) 0.5;
-    eps 	 = (T) 0.0;
+    del    = (T) 0.5;
+    eps    = (T) 0.0;
     neweps = (T) 1.0;
-	
+  
     while ( del > 0 ) {
-      if ( 1 + neweps > 1 ) {	/* Then the value might be too large */
-	eps = neweps;		/* ...save the current value... */
-	neweps -= del;		/* ...and decrement a bit */
-      } else {			/* Then the value is too small */
-	neweps += del;		/* ...so increment it */
+      if ( 1 + neweps > 1 ) {  /* Then the value might be too large */
+        eps = neweps;    /* ...save the current value... */
+        neweps -= del;    /* ...and decrement a bit */
+      } else {      /* Then the value is too small */
+        neweps += del;    /* ...so increment it */
       }
-      del *= 0.5;			/* Reduce the adjustment by half */
+      del *= 0.5;      /* Reduce the adjustment by half */
     }
 
     return eps;
@@ -103,21 +114,21 @@ namespace SCYTHE {
   {
     if (a > b)
       throw scythe_invalid_arg (__FILE__, __PRETTY_FUNCTION__, __LINE__,
-				"Lower limit larger than upper");
-		
+        "Lower limit larger than upper");
+    
     if (N <= 0)
       throw scythe_invalid_arg (__FILE__, __PRETTY_FUNCTION__, __LINE__,
-				"Number of subintervals negative");
+        "Number of subintervals negative");
 
     T I = (T) 0;
     T w = (b - a) / N;
     for (int i = 1; i <= N; i++)
       I += w * (fun(a +(i - 1) *w) + 4 * fun(a - w / 2 + i * w) +
-		fun(a + i * w)) / 6;
+          fun(a + i * w)) / 6;
    
     return I;
   }
-	
+  
   /* Calculate the definite integral of a function from a to b
    * Notes: Algorithm taken from Sedgewick, Robert. 1992. Algorithms
    * in C++. Addison Wesley. pg. 562
@@ -125,20 +136,20 @@ namespace SCYTHE {
   template <class T>
   T
   adaptsimp(T (*fun)(const T &), const T &a, const T &b, const int &N,
-	    const T tol)
+      const T tol)
   {
     if (a > b)
       throw scythe_invalid_arg (__FILE__, __PRETTY_FUNCTION__, __LINE__,
-				"Lower limit larger than upper");
-		
+        "Lower limit larger than upper");
+    
     if (N <= 0)
       throw scythe_invalid_arg (__FILE__, __PRETTY_FUNCTION__, __LINE__,
-				"Number of subintervals negative");
+        "Number of subintervals negative");
 
     T I = intsimp(fun, a, b, N);
     if (::fabs(I - intsimp(fun, a, b, N / 2)) > tol)
       return adaptsimp(fun, a, (a + b) / 2, N, tol)
-	+ adaptsimp(fun, (a + b) / 2, b, N, tol);
+        + adaptsimp(fun, (a + b) / 2, b, N, tol);
 
     return I;
   }
@@ -151,12 +162,12 @@ namespace SCYTHE {
   template <class T>
   Matrix<T>
   gradfdif (T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		     const Matrix<T> &), const Matrix<T> &theta,
-	    const Matrix<T> &y, const Matrix<T> &X)
+         const Matrix<T> &), const Matrix<T> &theta,
+      const Matrix<T> &y, const Matrix<T> &X)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
 
     int k = theta.size();
     // stepsize CAREFUL-- THIS IS MACHINE-SPECIFIC!!!
@@ -186,16 +197,16 @@ namespace SCYTHE {
   template <class T>
   T
   gradfdifls (T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		       const Matrix<T> &),  const T &alpha,
-	      const Matrix<T> &theta, const Matrix<T> &p, 
-	      const Matrix<T> &y, const Matrix<T> &X)
+           const Matrix<T> &),  const T &alpha,
+        const Matrix<T> &theta, const Matrix<T> &p, 
+        const Matrix<T> &y, const Matrix<T> &X)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
     if (! p.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "p not column vector");
+            __LINE__, "p not column vector");
 
     int k = theta.size();
     // stepsize CAREFUL-- THIS IS MACHINE-SPECIFIC!!!
@@ -209,9 +220,9 @@ namespace SCYTHE {
       donothing(temp);
       T e = temp - alpha;
       deriv = (fun(theta + (alpha + e) * p, y, X)
-	       - fun(theta + alpha * p, y, X)) / e;
+         - fun(theta + alpha * p, y, X)) / e;
     }
-		
+    
     return deriv;
   }
 
@@ -227,7 +238,7 @@ namespace SCYTHE {
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
    
     Matrix<T> fval = fun(theta);
 
@@ -247,7 +258,7 @@ namespace SCYTHE {
       Matrix<T> fthetae = fun(theta + e);
       Matrix<T> ftheta = fun(theta);
       for (int j = 0; j < n; ++j) {
-	J(j,i) = (fthetae[j] - ftheta[j]) / e[i];
+        J(j,i) = (fthetae[j] - ftheta[j]) / e[i];
       }
     }
    
@@ -261,14 +272,14 @@ namespace SCYTHE {
   template <class T>
   Matrix<T>
   jacfdif(Matrix<T> (*fun)(const Matrix<T> &, const Matrix<T> &), 
-	  const Matrix<T> &theta, const Matrix<T> &psi)
+    const Matrix<T> &theta, const Matrix<T> &psi)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
    
     Matrix<T> fval = fun(theta, psi);
-		
+    
     int k = theta.rows();
     int n = fval.rows();
     // stepsize CAREFUL -- THIS IS MACHINE-SPECIFIC!!!
@@ -285,7 +296,7 @@ namespace SCYTHE {
       Matrix<T> fthetae = fun(theta + e, psi);
       Matrix<T> ftheta = fun(theta, psi);
       for (int j = 0; j < n; ++j) {
-	J(j,i) = (fthetae[j] - ftheta[j]) / e[i];
+        J(j,i) = (fthetae[j] - ftheta[j]) / e[i];
       }
     }
    
@@ -296,13 +307,13 @@ namespace SCYTHE {
   template <class T>
   Matrix<T>
   hesscdif (T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		     const Matrix<T> &), const Matrix<T> &theta,
-	    const Matrix<T> &y, const Matrix<T> &X)
+         const Matrix<T> &), const Matrix<T> &theta,
+      const Matrix<T> &y, const Matrix<T> &X)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
-		
+            __LINE__, "Theta not column vector");
+    
     T fval = fun(theta,y,X);
 
     int k = theta.rows();
@@ -319,25 +330,25 @@ namespace SCYTHE {
       donothing(temp);
       ei = temp - theta;
       for (int j = 0; j < k; ++j){
-	Matrix<T> ej(k,1);
-	ej[j] = h;
-	temp = theta + ej;
-	donothing(temp);
-	ej = temp - theta;
-				
-	if (i==j){
-	  H(i,i) = ( -fun(theta + 2.0 * ei, y, X) + 16.0 * fun(theta+ei,y,X)
-		     - 30.0 * fval + 16.0 * fun(theta-ei,y,X)
-		     - fun(theta-2.0 * ei, y, X)) / 
-	             (12.0 * h2);
-	} else {
-	  H(i,j) = ( fun(theta + ei + ej, y, X) - fun(theta+ei-ej, y, X)
-		     - fun(theta - ei + ej, y, X) + fun(theta-ei-ej, y, X))
-	           / (4.0 * h2);
-	}
+        Matrix<T> ej(k,1);
+        ej[j] = h;
+        temp = theta + ej;
+        donothing(temp);
+        ej = temp - theta;
+        
+        if (i==j){
+          H(i,i) = ( -fun(theta + 2.0 * ei, y, X) + 16.0 *
+              fun(theta+ei,y,X) - 30.0 * fval + 16.0 *
+              fun(theta-ei,y,X) -
+              fun(theta-2.0 * ei, y, X)) / (12.0 * h2);
+        } else {
+          H(i,j) = ( fun(theta + ei + ej, y, X) - fun(theta+ei-ej, y, X)
+              - fun(theta - ei + ej, y, X) + fun(theta-ei-ej, y, X))
+            / (4.0 * h2);
+        }
       }
     }
-  		 
+       
     return H;
   }
 
@@ -348,16 +359,16 @@ namespace SCYTHE {
   template <class T>
   T
   linesearch1(T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		       const Matrix<T> &), const Matrix<T> &theta,
-	      const Matrix<T> &p, const Matrix<T> &y,
-	      const Matrix<T> &X)
+           const Matrix<T> &), const Matrix<T> &theta,
+        const Matrix<T> &p, const Matrix<T> &y,
+        const Matrix<T> &X)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
     if (! p.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "p not column vector");
+            __LINE__, "p not column vector");
 
     T alpha_bar = (T) 1.0;
     T rho = (T) 0.9;
@@ -366,7 +377,7 @@ namespace SCYTHE {
     Matrix<T> fgrad = gradfdif(fun, theta, y, X);
 
     while (fun(theta + alpha * p, y, X) > (fun(theta, y, X) + c
-					   * alpha * t(fgrad) * p)[0]) {
+             * alpha * t(fgrad) * p)[0]) {
       alpha = rho * alpha;
     }
 
@@ -380,16 +391,16 @@ namespace SCYTHE {
   template <class T>
   T
   linesearch2(T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		       const Matrix<T> &), const Matrix<T> &theta,
-	      const Matrix<T> &p, const Matrix<T> &y,
-	      const Matrix<T> &X)
+           const Matrix<T> &), const Matrix<T> &theta,
+        const Matrix<T> &p, const Matrix<T> &y,
+        const Matrix<T> &X)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
     if (! p.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "p not column vector");
+            __LINE__, "p not column vector");
 
     T alpha_last = (T) 0.0;
     T alpha_cur = (T) 1.0;
@@ -404,21 +415,21 @@ namespace SCYTHE {
       T phi_last = fun(theta + alpha_last * p, y, X);
      
       if ((phi_cur > (fun(theta, y, X) + c1 * alpha_cur * fgradalpha0))
-	  ||
-	  ((phi_cur >= phi_last) && (i > 0))) {
-	T alphastar = zoom(fun, alpha_last, alpha_cur, theta, p, y, X);
-	return alphastar;
+          ||
+          ((phi_cur >= phi_last) && (i > 0))) {
+        T alphastar = zoom(fun, alpha_last, alpha_cur, theta, p, y, X);
+        return alphastar;
       }
 
       T fgradalpha_cur = gradfdifls(fun, alpha_cur, theta, p, y, X);
       if ( ::fabs(fgradalpha_cur) <= -1 * c2 * fgradalpha0)
-	return alpha_cur;
+        return alpha_cur;
 
       if ( fgradalpha_cur >= (T) 0.0) {
-	T alphastar = zoom(fun, alpha_cur, alpha_last, theta, p, y, X);
-	return alphastar;
+        T alphastar = zoom(fun, alpha_cur, alpha_last, theta, p, y, X);
+        return alphastar;
       }
-			
+      
       alpha_last = alpha_cur;
       alpha_cur = runif() * (alpha_max - alpha_cur) + alpha_cur;
     }
@@ -435,16 +446,16 @@ namespace SCYTHE {
   template <class T>
   T
   zoom (T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		 const Matrix<T> &), const T &alo, const T &ahi,
-	const Matrix<T> &theta, const Matrix<T> &p,
-	const Matrix<T> &y, const Matrix<T> &X )
+        const Matrix<T> &), const T &alo, const T &ahi,
+        const Matrix<T> &theta, const Matrix<T> &p,
+        const Matrix<T> &y, const Matrix<T> &X )
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
     if (! p.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "p not column vector");
+            __LINE__, "p not column vector");
 
     T alpha_lo = alo;
     T alpha_hi = ahi;
@@ -460,18 +471,18 @@ namespace SCYTHE {
       T phi_j = fun(theta + alpha_j * p, y, X);
       T phi_lo = fun(theta + alpha_lo * p, y, X);
      
-      if ((phi_j > (phi_0 + c1 * alpha_j * fgrad0)) ||
-	  (phi_j >= phi_lo)){
-	alpha_hi = alpha_j;
+      if ((phi_j > (phi_0 + c1 * alpha_j * fgrad0))
+          || (phi_j >= phi_lo)){
+        alpha_hi = alpha_j;
       } else {
-	T fgradj = gradfdifls(fun, alpha_j, theta, p, y, X);
-	if (::fabs(fgradj) <= -1 * c2 * fgrad0){
-	  return alpha_j;
-	}
-	if ( fgradj * (alpha_hi - alpha_lo) >= 0){
-	  alpha_hi = alpha_lo;
-	}
-	alpha_lo = alpha_j;
+        T fgradj = gradfdifls(fun, alpha_j, theta, p, y, X);
+        if (::fabs(fgradj) <= -1 * c2 * fgrad0){ 
+          return alpha_j;
+        }
+        if ( fgradj * (alpha_hi - alpha_lo) >= 0){
+          alpha_hi = alpha_lo;
+        }
+        alpha_lo = alpha_j;
       }
       ++count;
     }
@@ -488,14 +499,14 @@ namespace SCYTHE {
   template <class T>
   Matrix<T>
   BFGS (T (*fun)(const Matrix<T> &, const Matrix<T> &,
-		 const Matrix<T> &), const Matrix<T> &theta, 
-	const Matrix<T> &y, const Matrix<T> &X, const int &maxit = 1000,
-	const T &tolerance = 1e-6)
+     const Matrix<T> &), const Matrix<T> &theta, 
+  const Matrix<T> &y, const Matrix<T> &X, const int &maxit = 1000,
+  const T &tolerance = 1e-6)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
-		
+            __LINE__, "Theta not column vector");
+    
     int n = theta.size();
 
     // H is initial inverse hessian
@@ -517,7 +528,7 @@ namespace SCYTHE {
       Matrix<T> y = fgrad_new - fgrad;
       T rho = 1.0 / (t(y) * s)[0];
       H = (I - rho * s * t(y)) * H *(I - rho * y * t(s))
-	+ rho * s * (!s);
+        + rho * s * (!s);
 
       thetamin = thetamin_new;
       fgrad = fgrad_new;
@@ -527,11 +538,11 @@ namespace SCYTHE {
       std::cout << "thetamin = " << (!thetamin).toString() << std::endl;
       std::cout << "gradient = " << (!fgrad).toString() << std::endl;
       std::cout << "t(gradient) * gradient = " << 
-	((!fgrad) * fgrad).toString() << std::endl;
+        ((!fgrad) * fgrad).toString() << std::endl;
 
       if (count > maxit)
-	throw scythe_convergence_error (__FILE__, __PRETTY_FUNCTION__,
-					__LINE__, "Failed to converge.  Try better starting values");
+        throw scythe_convergence_error (__FILE__, __PRETTY_FUNCTION__,
+          __LINE__, "Failed to converge.  Try better starting values");
     }
    
     return thetamin;
@@ -546,12 +557,12 @@ namespace SCYTHE {
   template <class T>
   Matrix<T>
   nls_broyden(Matrix<T> (*fun)(const Matrix<T> &),
-	      const Matrix<T> &theta,  const int &maxit = 5000,
-	      const T &tolerance = 1e-6)
+        const Matrix<T> &theta,  const int &maxit = 5000,
+        const T &tolerance = 1e-6)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
 
     Matrix<T> thetastar = theta;
     Matrix<T> B = jacfdif(fun, thetastar);
@@ -574,12 +585,12 @@ namespace SCYTHE {
       B = B + ((y - B * s) * (!s)) / ((!s) * s);
       thetastar = thetastar_new;
       if (max(fabs(fthetastar_new)) < tolerance)
-	return thetastar;
+        return thetastar;
     }
-		
+    
     throw scythe_convergence_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__,std::string("Failed to converge.  Try better starting") &
-				    " values or increase maxit");
+      __LINE__,std::string("Failed to converge.  Try better starting") &
+            " values or increase maxit");
   }
 
 
@@ -592,15 +603,15 @@ namespace SCYTHE {
   template <class T>
   Matrix<T>
   nls_broyden(Matrix<T> (*fun)(const Matrix<T> &, const Matrix<T> &), 
-	      const Matrix<T> &theta,  const Matrix<T> &psi, 
-	      const int &maxit=5000, const T &tolerance=1e-6)
+        const Matrix<T> &theta,  const Matrix<T> &psi, 
+        const int &maxit=5000, const T &tolerance=1e-6)
   {
     if (! theta.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__, "Theta not column vector");
+            __LINE__, "Theta not column vector");
     if (! psi.isColVector())
       throw scythe_dimension_error (__FILE__, __PRETTY_FUNCTION,
-				    __LINE__, "Psi not column vector");
+            __LINE__, "Psi not column vector");
 
     Matrix<T> thetastar = theta;
     Matrix<T> B = jacfdif(fun, thetastar, psi);
@@ -623,12 +634,12 @@ namespace SCYTHE {
       B = B + ((y - B * s) * (!s)) / ((!s) * s);
       thetastar = thetastar_new;
       if (max(fabs(fthetastar_new)) < tolerance)
-	return thetastar;
+        return thetastar;
     }
-		
+    
     throw scythe_convergence_error (__FILE__, __PRETTY_FUNCTION__,
-				    __LINE__,std::string("Failed to converge.  Try better starting") &
-				    " values or increase maxit");
+      __LINE__,std::string("Failed to converge.  Try better starting") &
+            " values or increase maxit");
   }
 
 }  // namespace dec
